const { PLOTS_DIR } = require('./config');
const chalk = require('chalk');
const { version, author } = require('../package.json');
const commandLineArgs = require('command-line-args');

const plotter = require('./plotter');
const { create: createPlotPartition } = require('./plotPartition');
const { hasAdminPrivileges } = require('./privilege');
const ui = require('./ui');
const cache = require('./cache');
const fs = require('fs-extra');

const isDevMode = process.env.NODE_ENV === 'development';

const options = commandLineArgs([{ name: 'cache', alias: 'c', type: String }, { name: 'extended', alias: 'e', type: Boolean }]);

(function run() {

	console.log('\n');
	console.log(chalk`{whiteBright --------------------------------------------------}`);
	console.log(chalk`{blueBright.bold BURST Auto Plotter} based on XPlotter`);
	console.log(chalk`Version {whiteBright ${version}}`);
	console.log(`by ${author.name}`);
	if (options.extended) {
		console.log('\n');
		console.log(chalk`{whiteBright Extended Mode Activated}`);
		console.log('\n');
	}
	console.log(chalk`{whiteBright --------------------------------------------------}`);
	console.log('\n');

	if (!isDevMode && !hasAdminPrivileges()) {
		console.log(chalk`🚸 {redBright No Admin rights!}`);
		console.log('For significantly faster writes you should run autoplotter as administrator');
		process.exit(666);
		return;
	}

	ui.run(cache.load(options.cache), { extended: options.extended }).then(answers => {
		cache.update(answers, options.cache);
		return answers;
	}).then(answers => {

		try {
			const { accountId, hardDisk, startNonce, totalPlotSize, chunks, threads, memory } = answers;
			const path = `${hardDisk}:/${PLOTS_DIR}`;

			fs.ensureDirSync(path);

			const { totalNonces, plots } = createPlotPartition(totalPlotSize, startNonce, chunks);

			const lastPlot = plots[plots.length - 1];
			cache.update({ lastNonce: lastPlot.startNonce + lastPlot.nonces }, options.cache);

			console.log(chalk`Created partition for {whiteBright ${totalPlotSize} GiB} in {whiteBright ${chunks} chunk(s)}`);
			console.log(chalk`Overall nonces to be written: {whiteBright ${totalNonces}}`);
			console.log(chalk`Threads used: {whiteBright ${threads}}`);
			console.log(chalk`Memory used: {whiteBright ${memory}MiB}`);

			plotter.start({
				totalNonces,
				plots,
				accountId,
				path,
				threads,
				memory
			});
		} catch (e) {
			console.error(`Couldn't create directory ${path} - reason: ${e}`);
			process.exit(666);
		}
	});
})();